﻿using System;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using Org.BouncyCastle.Crypto;
using Org.BouncyCastle.Crypto.Parameters;
using Org.BouncyCastle.Crypto.Engines;
using Org.BouncyCastle.Crypto.Encodings;

namespace fds.bestin.util
{
    public class RSAUtil
    {

        /// <summary>   
        /// RSA decrypt   
        /// </summary>   
        /// <param name="publicKey"></param>   
        /// <param name="content"></param>   
        /// <returns></returns>   
        public static string dencrypt(string privateKey, string password, string encrypt_data)
        {
            X509Certificate2 x509Cert = new X509Certificate2(privateKey, password, X509KeyStorageFlags.Exportable);
            RSACryptoServiceProvider _RSA = (RSACryptoServiceProvider)x509Cert.PrivateKey;
            RSAParameters parameters = _RSA.ExportParameters(true);

            AsymmetricKeyParameter bouncyCastlePrivateKey = new RsaPrivateCrtKeyParameters(
                                                                  new Org.BouncyCastle.Math.BigInteger(1, parameters.Modulus),
                                                                  new Org.BouncyCastle.Math.BigInteger(1, parameters.Exponent),
                                                                  new Org.BouncyCastle.Math.BigInteger(1, parameters.D),
                                                                  new Org.BouncyCastle.Math.BigInteger(1, parameters.P),
                                                                  new Org.BouncyCastle.Math.BigInteger(1, parameters.Q),
                                                                  new Org.BouncyCastle.Math.BigInteger(1, parameters.DP),
                                                                  new Org.BouncyCastle.Math.BigInteger(1, parameters.DQ),
                                                                  new Org.BouncyCastle.Math.BigInteger(1, parameters.InverseQ));
            byte[] encryptData = Convert.FromBase64String(encrypt_data);
            var cipher = new Pkcs1Encoding(new RsaEngine());
            cipher.Init(false, bouncyCastlePrivateKey);//false表示解密
            var blockSize = cipher.GetInputBlockSize();
            byte[] _data = cipher.ProcessBlock(encryptData, 0, encryptData.Length);
            
            return Encoding.UTF8.GetString(_data);
        }

        /// <summary>   
        /// RSA encrypt   
        /// </summary>   
        /// <param name="publicKey"></param>  
        /// <returns></returns>   
        public static string encrypt(string publicKey, string data)
        {
            X509Certificate2 x509Cert = new X509Certificate2(publicKey);
            RSACryptoServiceProvider _RSA = (RSACryptoServiceProvider)x509Cert.PublicKey.Key;
            RSAParameters parameters = _RSA.ExportParameters(false);

            AsymmetricKeyParameter pubKey = new RsaKeyParameters(false, new Org.BouncyCastle.Math.BigInteger(1, parameters.Modulus), new Org.BouncyCastle.Math.BigInteger(1, parameters.Exponent));

            var cipher = new Pkcs1Encoding(new RsaEngine());
            cipher.Init(true, pubKey);//true表示加密 
            var blockSize = cipher.GetInputBlockSize();
            byte[] _data = Encoding.UTF8.GetBytes(data);
            byte[] encryptData = cipher.ProcessBlock(_data, 0, _data.Length);

            return Convert.ToBase64String(encryptData);
        }
    }
}