package demo;

import com.alibaba.fastjson.JSONObject;
import demo.comm.DSAUtil;
import demo.comm.RSAUtil;
import demo.comm.SOAClient;

import java.security.interfaces.RSAPublicKey;

/**
 * SOA接口调用示例
 *
 * @date: 2020/06/23
 **/
public class SoaDemo {

    private final static String URL = "http://xxxxxxxx/service/soa";
    private final static String VERSION = "3.0";
    private final static String APP_ID = "";
    private final static String SERVICE = "member.create";

    /**
     * 调用keyToString方法将密钥转换为字符串
     */
    private final static String DSA_PRIVATE_KEY = "";

    private final static String CLOUD_DSA_PUBLIC_KEY = "";
    private final static String CLOUD_RSA_PUBLIC_KEY = "";

    public static void main(String[] args) throws Exception {

        SOAClient soaClient = new SOAClient();

        //应用号，钱账通提供的应用ID
        soaClient.setAppId(APP_ID);
        //版本号，当前版本为 3.0
        soaClient.setVersion(VERSION);
        //商户DSA私钥，钱账通提供的DSA私钥
        soaClient.setPrivateKey(DSAUtil.getPrivateKey(DSA_PRIVATE_KEY));
        //钱账通DSA公钥，钱账通提供的DSA公钥
        soaClient.setPublicKey(DSAUtil.getPublicKey(CLOUD_DSA_PUBLIC_KEY));
        //钱账通接口地址
        soaClient.setServerUrl(URL);

        //请求参数中身份证号、银行卡号等私密信息 使用钱账通提供的RSA公钥 进行加密
        RSAPublicKey rsaPublicKey = RSAUtil.getPublicKey(CLOUD_RSA_PUBLIC_KEY);
        String enIdentityNo = RSAUtil.encrypt(rsaPublicKey, "110101xxxx03077395");

        JSONObject param = new JSONObject();
        param.put("out_member_no", System.currentTimeMillis() + "");
        param.put("member_type", "2");
        param.put("identity_no", enIdentityNo);
        param.put("identity_name", "test");
        param.put("identity_type", 1);
        JSONObject response = soaClient.request(SERVICE, param);
        System.out.println("response:" + response);
    }


    private static void keyToString() throws Exception {

//        DSA公私钥转换字符串方法
//        System.out.println(DSAUtil.getPrivateKeyString((DSAPrivateKey) DSAUtil.loadPrivateKey("dsa_private_key.pfx路径", "私钥密码")));
//        System.out.println(DSAUtil.getPublicKeyString((DSAPublicKey) DSAUtil.loadPublicKey("dsa_public_key.cer公钥路径")));

//        RSA公私钥转换字符串方法
//        System.out.println(RSAUtil.getPrivateKeyString((RSAPrivateKey) RSAUtil.loadPrivateKey("rsa_private_key.pfx私钥路径", "私钥密码")));
//        System.out.println(RSAUtil.getPublicKeyString((RSAPublicKey) RSAUtil.loadPublicKey("rsa_public_key.cer公钥路径")));

    }
}
