package demo;

import com.alibaba.fastjson.JSONObject;
import demo.comm.DSAUtil;
import org.apache.commons.io.FileUtils;

import java.io.File;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.security.PrivateKey;
import java.util.HashMap;
import java.util.Map;

/**
 * @classname: SoaDownloadCheckFileDemo
 * @description: soa下载对账文件demo
 * @author: BiJiacheng
 * @create: 2021/5/8 2:29 下午
 **/

public class SoaDownloadCheckFileDemo {

    private final static String URL = "http://xxxxxxxx/order/service/download";
    private final static String VERSION = "3.0";
    private final static String APP_ID = "";
    private final static String SERVICE = "order.check.download";
    private final static String DSA_PRIVATE_KEY = "";

    public static void main(String[] args) throws Exception {
        JSONObject param = new JSONObject();
        param.put("bill_date", "2021-05-07");

        String timestamp = String.valueOf(System.currentTimeMillis() / 1000);

        Map<String, String> req = new HashMap<>();
        req.put("params", param.toJSONString());
        req.put("app_id", APP_ID);
        req.put("version", VERSION);
        req.put("service", SERVICE);
        req.put("timestamp", timestamp);
        PrivateKey privateKey = DSAUtil.getPrivateKey(DSA_PRIVATE_KEY);
        String sign = DSAUtil.sign(privateKey, APP_ID + timestamp + VERSION + SERVICE + param.toJSONString());
        req.put("sign", sign);

        InputStream inputStream = request(URL, req);
        FileUtils.copyInputStreamToFile(inputStream, new File("/temp/test.zip"));
    }

    private static InputStream request(String serverUrl, Map<String, String> args) throws Exception {
        java.net.URL url = new URL(serverUrl);
        // Open a connection to the URL
        HttpURLConnection connection = (HttpURLConnection) url.openConnection();

        connection.setRequestMethod("GET");
        connection.setAllowUserInteraction(false);
        connection.setInstanceFollowRedirects(false);
        connection.setUseCaches(false);
        // 设置 HttpURLConnection的字符编码
        connection.setRequestProperty("content-type", "application/x-www-form-urlencoded;charset=UTF-8");
        StringBuffer sb = new StringBuffer();
        byte[] reqbody = null;
        for (Map.Entry<String, String> entry : args.entrySet()) {
            sb.append(entry.getKey()).append("=").append(URLEncoder.encode(entry.getValue(), "UTF-8")).append("&");
        }
        sb.append("\r\n");
        reqbody = sb.toString().getBytes();
        connection.addRequestProperty("Content-length", Integer.toString(reqbody.length));
        // Set the input/output state
        connection.setDoInput(true);
        connection.setDoOutput(true);
        // Connect to the server
        connection.connect();
        // Write the request body
        int len;
        if (connection.getDoOutput()) {
            OutputStream outputStream = null;
            try {
                outputStream = connection.getOutputStream();
                outputStream.write(reqbody);
            } finally {
                if (outputStream != null) {
                    outputStream.close();
                }
            }
        }
        int status = connection.getResponseCode();
        if (status != 200) {
            throw new Exception("下载文件失败");
        }

        return connection.getInputStream();
    }

}
