package demo.comm;

import org.apache.commons.lang.StringUtils;

import javax.net.ssl.*;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.security.cert.X509Certificate;
import java.util.List;

/**
 * Created by HeLong on 2017/7/28.
 */
public class HttpPostEmulator {
    // 每个post参数之间的分隔。随意设定，只要不会和其他的字符串重复即可。
    private static final String BOUNDARY = "----------HV2ymHFg03ehbqgZCaKO6jyH";


    private static class TrustAnyTrustManager implements X509TrustManager {
        @Override
        public X509Certificate[] getAcceptedIssuers() {
            X509Certificate[] x509Certificates = new X509Certificate[]{};
            return x509Certificates;
        }

        public void checkClientTrusted(X509Certificate[] chain,
                                       String authType)
                throws java.security.cert.CertificateException {
        }

        public void checkServerTrusted(X509Certificate[] chain,
                                       String authType)
                throws java.security.cert.CertificateException {
        }
    }

    private static class TrustAnyHostnameVerifier implements HostnameVerifier {
        public boolean verify(String hostname, SSLSession session) {
            return true;
        }
    }

    /**
     * 发送https请求(正式)
     *
     * @param serverUrl
     * @param generalFormFields
     * @param filesToBeUploaded
     * @return
     * @throws Exception
     */
    public String sendHttpsPostRequest(String serverUrl,
                                       List<FormFieldKeyValuePair> generalFormFields,
                                       List<UploadFileItem> filesToBeUploaded,
                                       String charset) throws Exception {
        if (StringUtils.isBlank(charset)) {
            charset = "GBK";
        }
        // 向服务器发送post请求
        URL url = new URL(serverUrl);
        SSLContext sc = SSLContext.getInstance("SSL");
        sc.init(null, new X509TrustManager[]{new TrustAnyTrustManager()}, new java.security.SecureRandom());
        HttpsURLConnection connection = (HttpsURLConnection) url.openConnection();
        connection.setSSLSocketFactory(sc.getSocketFactory());
        connection.setHostnameVerifier(new TrustAnyHostnameVerifier());

        // 发送POST请求必须设置如下两行
        connection.setDoOutput(true);
        connection.setDoInput(true);
        connection.setUseCaches(false);
        connection.setRequestMethod("POST");
        connection.setRequestProperty("Connection", "Keep-Alive");
        connection.setRequestProperty("Charset", charset);
        connection.setRequestProperty("Content-Type",
                "multipart/form-data; boundary=" + BOUNDARY);
        connection.connect();
        // 传输内容
        StringBuffer contentBody = new StringBuffer("--" + BOUNDARY);
        // 尾
        String endBoundary = "\r\n--" + BOUNDARY + "--\r\n";
        OutputStream out = connection.getOutputStream();
        // 1. 处理文字形式的POST请求
        contentBody = dealFormFields(contentBody, generalFormFields);
        String formFieldMessage = contentBody.toString();
        out.write(formFieldMessage.getBytes(charset));
        // 2. 处理文件上传
        dealImageFile(filesToBeUploaded, out, charset);
        out.write("------------HV2ymHFg03ehbqgZCaKO6jyH--\r\n"
                .getBytes(charset));
        // 3. 写结尾
        out.write(endBoundary.getBytes(charset));
        out.flush();
        out.close();
        // 4. 从服务器获得回答的内容
        String strLine = "";
        String strResponse = "";
        InputStream in = connection.getInputStream();
        BufferedReader reader = new BufferedReader(new InputStreamReader(in));
        while ((strLine = reader.readLine()) != null) {
            strResponse += strLine + "\n";
        }
        return strResponse;
    }

    /**
     * 发送http请求测试
     *
     * @param serverUrl
     * @param generalFormFields
     * @param filesToBeUploaded
     * @return
     * @throws Exception
     */
    public String sendHttpPostRequest(String serverUrl,
                                      List<FormFieldKeyValuePair> generalFormFields,
                                      List<UploadFileItem> filesToBeUploaded,
                                      String charset) throws Exception {
        if (StringUtils.isBlank(charset)) {
            charset = "GBK";
        }
        // 向服务器发送post请求
        URL url = new URL(serverUrl);
        HttpURLConnection connection = (HttpURLConnection) url.openConnection();
        // 发送POST请求必须设置如下两行
        connection.setDoOutput(true);
        connection.setDoInput(true);
        connection.setUseCaches(false);
        connection.setRequestMethod("POST");
        connection.setRequestProperty("Connection", "Keep-Alive");
        connection.setRequestProperty("Charset", charset);
        connection.setRequestProperty("Content-Type",
                "multipart/form-data; boundary=" + BOUNDARY);
        connection.connect();
        // 传输内容
        StringBuffer contentBody = new StringBuffer("--" + BOUNDARY);
        // 尾
        String endBoundary = "\r\n--" + BOUNDARY + "--\r\n";
        OutputStream out = connection.getOutputStream();
        // 1. 处理文字形式的POST请求
        contentBody = dealFormFields(contentBody, generalFormFields);
        String formFieldMessage = contentBody.toString();
        out.write(formFieldMessage.getBytes(charset));
        // 2. 处理文件上传
        dealImageFile(filesToBeUploaded, out, charset);
        out.write("------------HV2ymHFg03ehbqgZCaKO6jyH--\r\n"
                .getBytes(charset));

        // 3. 写结尾
        out.write(endBoundary.getBytes(charset));
        out.flush();
        out.close();
        // 4. 从服务器获得回答的内容
        String strLine = "";
        String strResponse = "";
        InputStream in = connection.getInputStream();
        BufferedReader reader = new BufferedReader(new InputStreamReader(in));
        while ((strLine = reader.readLine()) != null) {
            strResponse += strLine + "\n";
        }
        return strResponse;

    }

    private StringBuffer dealFormFields(StringBuffer contentBody, List<FormFieldKeyValuePair> generalFormFields) throws Exception {
        for (FormFieldKeyValuePair ffkvp : generalFormFields) {
            contentBody.append("\r\n")
                    .append("Content-Disposition: form-data; name=\"")
                    .append(ffkvp.getKey() + "\"")
                    .append("\r\n")
                    .append("\r\n")
                    .append(ffkvp.getValue())
                    .append("\r\n")
                    .append("--")
                    .append(BOUNDARY);
        }
        return contentBody;
    }

    private void dealImageFile(List<UploadFileItem> filesToBeUploaded, OutputStream out, String charset) throws Exception {
        //处理文件上传
        for (UploadFileItem ufi : filesToBeUploaded) {
            StringBuffer contentBody = new StringBuffer();
            contentBody.append("\r\n")
                    .append("Content-Disposition:form-data; name=\"")
                    .append(ufi.getFormFieldName() + "\"; ") // form中field的名称
                    .append("filename=\"")
                    .append(ufi.getFileName() + "\"") // 上传文件的文件名，包括目录
                    .append("\r\n")
                    .append("Content-Type:image/jpeg")
                    .append("\r\n\r\n");
            String boundaryMessage2 = contentBody.toString();
            out.write(boundaryMessage2.getBytes(charset));
            // 开始真正向服务器写文件
            File file = new File(ufi.getFileName());
            DataInputStream dis = new DataInputStream(new FileInputStream(file));
            int bytes = 0;
            byte[] bufferOut = new byte[(int) file.length()];
            bytes = dis.read(bufferOut);
            out.write(bufferOut, 0, bytes);
            dis.close();
            contentBody.append("------------HV2ymHFg03ehbqgZCaKO6jyH");
            String boundaryMessage = contentBody.toString();
            out.write(boundaryMessage.getBytes(charset));
        }
    }
}
